// Programming 2D Games
// Copyright (c) 2011 by: 
// Charles Kelly
// ship.cpp v1.0


#include "EntityMonster.h"
using namespace EntityMonsterNS;

//=============================================================================
// default constructor
//=============================================================================
EntityMonster::EntityMonster() : Entity()
{
	spriteData.width = WIDTH;           // size of EntityPlayer
	spriteData.height = HEIGHT;
	spriteData.x = X;                   // location on screen
	spriteData.y = Y;
	spriteData.rect.bottom = HEIGHT;    // rectangle to select parts of an image
	spriteData.rect.right = WIDTH;
	edge.top    = 0;             // ROTATED_BOX collision edges
	edge.bottom = 32;
	edge.left   = 0;
	edge.right  = 32;
	velocity.x = 0;
	velocity.y = 0;
	frameDelay = 0.3f;
	startFrame = 0;     // first frame of animation
	endFrame     = 3;     // last frame of animation
	currentFrame = startFrame;
	radius = WIDTH/2.0;
	collisionType = entityNS::BOX;
	QueryPerformanceCounter(&lastAttack);
	QueryPerformanceFrequency(&tmrFreq);
	yOffset = 0;
	xOffset = 0;
	xMoving = 0;
	yMoving = 0;
	movement = 0;
	health = 100;
}

//=============================================================================
// Initialize the EntityPlayer.
// Post: returns true if successful, false if failed
//=============================================================================
bool EntityMonster::initialize(Game *gamePtr, int width, int height, int ncols,
							   TextureManager *textureM)
{
	return(Entity::initialize(gamePtr, width, height, ncols, textureM));
}

//=============================================================================
// update
// typically called once per frame
// frameTime is used to regulate the speed of movement and animation
//=============================================================================
void EntityMonster::update(float frameTime)
{
	Entity::update(frameTime);

	if (xMoving != 0) {
		xMoving -= 1;
		xOffset += movement;
	}
	if (yMoving != 0) {
		yMoving -= 1;
		yOffset += movement;
	}
	if (xMoving == 0 && yMoving == 0) {
		if (xOffset < 0)
			setX(getX() - 1);
		if (xOffset > 0)
			setX(getX() + 1);
		if (yOffset < 0)
			setY(getY() - 1);
		if (yOffset > 0)
			setY(getY() + 1);
		xOffset = 0;
		yOffset = 0;
	}
}

bool EntityMonster::isMoving() {
	return xMoving != 0 || yMoving != 0;
}

int EntityMonster::getMonsterX() {return spriteData.x;}
int EntityMonster::getMonsterY() {return spriteData.y;}

void EntityMonster::attackPlayer(EntityPlayer& thePlayer, int level) {
	//LARGE_INTEGER curTime;
	//QueryPerformanceCounter(&curTime);
	//int timePassed = static_cast<unsigned int>( (curTime.QuadPart - lastAttack.QuadPart) * 1000 / tmrFreq.QuadPart );
	//if (timePassed >= 500) {
	//	lastAttack = curTime;
	int temp = ((rand() % (level + ((STARTING_HEALTH + level * HEALTH_GAINED_PER_LEVEL) / MONSTER_DIFFICULTY))) - (thePlayer.getPlayerDef() / 10));
	if (temp == 0) {
		infochat::addChat("A bat missed!");
		//globalAudio::audio->playCue();
		return;
	}
	if (temp < 0) {
		infochat::addChat("A bats attack was absorbed by your armor!");
		//audio->playCue(armorblock);
		return;
	}
	infochat::addChat("A " + getName() + " hits you for " + std::to_string(temp) + "!");
	//audio->playCue(MvP);
	thePlayer.setHealth(thePlayer.getHealth() - temp);
	//}
}

//=============================================================================
// Draw Image using color as filter. Default color is WHITE.
//=============================================================================
//void EntityPlayer::draw(COLOR_ARGB color)
//{
//int flyingFrame = getCurrentFrame();
//setCurrentFrame(EntityPlayerNS::FLYING_BODY);
//Image::draw(color);
//setCurrentFrame(flyingFrame);
//    Image::draw(color);
//}