// Programming 2D Games
// Copyright (c) 2011 by: 
// Charles Kelly
// ship.cpp v1.0


#include "EntityPlayer.h"
using namespace EntityPlayerNS;

//=============================================================================
// default constructor
//=============================================================================
EntityPlayer::EntityPlayer() : Entity()
{
	spriteData.width = WIDTH;           // size of EntityPlayer
	spriteData.height = HEIGHT;
	spriteData.x = X;                   // location on screen
	spriteData.y = Y;
	spriteData.rect.bottom = HEIGHT;    // rectangle to select parts of an image
	spriteData.rect.right = WIDTH;
	edge.top    = 0;             // ROTATED_BOX collision edges
	edge.bottom = 32;
	edge.left   = 0;
	edge.right  = 32;
	velocity.x = 0;
	velocity.y = 0;
	frameDelay = 0.1f;
	startFrame = 0;     // first frame of animation
	endFrame     = 3;     // last frame of animation
	currentFrame = startFrame;
	radius = WIDTH/2.0;
	collisionType = entityNS::BOX;
	xMoving = 0;
	yMoving = 0;
	standingFrame = 0;
	worldX = 0;
	worldY = 0;
	lastDir = 0;
	level = 0;
	exp = 0;
	str = PLAYER_BASE_STR;
	money = 0;
	def = 0;
	QueryPerformanceCounter(&lastAttack);
	QueryPerformanceFrequency(&tmrFreq);
}

//=============================================================================
// Initialize the EntityPlayer.
// Post: returns true if successful, false if failed
//=============================================================================
bool EntityPlayer::initialize(Game *gamePtr, int width, int height, int ncols,
							  TextureManager *textureM)
{
	return(Entity::initialize(gamePtr, width, height, ncols, textureM));
}

//=============================================================================
// update
// typically called once per frame
// frameTime is used to regulate the speed of movement and animation
//=============================================================================
void EntityPlayer::update(float frameTime)
{
	Entity::update(frameTime);
	if (xMoving == 0 && yMoving == 0) {
		if (input->isKeyDown(UP_KEY) || input->isKeyDown(UP_KEY2))       // if up arrow
		{
			yMoving = MOVING_LENGTH;
			movement = WALKING_SPEED;
			standingFrame = 12;
			lastDir = DIR_UP;
		}
		else if (input->isKeyDown(DOWN_KEY) || input->isKeyDown(DOWN_KEY2))     // if decrease altitude
		{
			yMoving = MOVING_LENGTH;
			movement = -WALKING_SPEED;
			standingFrame = 0;
			lastDir = DIR_DOWN;
		}
		else if (input->isKeyDown(LEFT_KEY) || input->isKeyDown(LEFT_KEY2))     // if left
		{
			xMoving = MOVING_LENGTH;
			movement = WALKING_SPEED;
			standingFrame = 4;
			lastDir = DIR_LEFT;
		}
		else if (input->isKeyDown(RIGHT_KEY) || input->isKeyDown(RIGHT_KEY2))    // if right
		{
			xMoving = MOVING_LENGTH;
			movement = -WALKING_SPEED;
			standingFrame = 8;
			lastDir = DIR_RIGHT;
		}
		setCurrentFrame(standingFrame);
		setFrames(standingFrame,standingFrame+3); 
	} else {
		if (xMoving != 0) {
			xMoving -= 1;
			worldX += movement;
		}
		if (yMoving != 0) {
			yMoving -= 1;
			worldY += movement;
		}
		if (xMoving == 0 && yMoving == 0) {
			setFrames(standingFrame, standingFrame);
			setCurrentFrame(standingFrame);
			audio->playCue(footsteps);
			if (Ring.itemID == HEALING_RING.itemID) {
				static int walkCounter = 0;
				walkCounter++;
				if (walkCounter >= HEALING_RING_TICK_SPEED) {
					setHealth(getHealth() + 1);
					if (getHealth() > getMaxHealth()) {
						setHealth(getMaxHealth());
					}
					walkCounter = 0;
				}
			}
		}
	}
}
void EntityPlayer::stopMoving() {
	xMoving = 0;
	yMoving = 0; 
	setCurrentFrame(standingFrame); 
	setFrames(standingFrame, standingFrame); 
}

bool EntityPlayer::canAttack() {
	LARGE_INTEGER curTime;
	QueryPerformanceCounter(&curTime);
	int timePassed = static_cast<unsigned int>( (curTime.QuadPart - lastAttack.QuadPart) * 1000 / tmrFreq.QuadPart );
	if (timePassed >= 500) {
		lastAttack = curTime;
		return true;
	}
	return false;
}

void EntityPlayer::setLevel(int newLevel) { level = newLevel >= 0 ? newLevel : 0; }
void EntityPlayer::setExp(int newExp) { 
	if (newExp >= 0) {
		exp = newExp;
		if (exp >= getNextLevelExp()) {
			exp -= getNextLevelExp();
			level++;
			audio->playCue(levelUp);
			health = getMaxHealth();
		}
	}
}
void EntityPlayer::addExp(int newExp) { 
	if (Ring.itemID == EXP_RING.itemID)
		newExp *= 1.1;
	setExp(getExp() + newExp);
}

int EntityPlayer::getPlayerStr() { 
	int str = ((getPlayerBaseStr() - 30) + (rand() % 20));
	if (str < 0)
		str = 0;
	int str2 = str + (rand() % (getPlayerBaseStr() - str));
	return str > str2 ? str : str2; 
} //TODO: Change this to be more dynamic

