// Programming 2D Games
// Copyright (c) 2011 by: 
// Charles Kelly
// ship.h v1.0

#ifndef _ENTITYPLAYER_H            // Prevent multiple definitions if this 
#define _ENTITYPLAYER_H            // file is included in more than one place
#define WIN32_LEAN_AND_MEAN

#include "entity.h"
#include "constants.h"
#include <math.h>
#include "ItemInfo.h"

namespace EntityPlayerNS
{
    const int   WIDTH = 128;                // image width (each frame)
    const int   HEIGHT = 128;               // image height
    const int   X = GAME_WIDTH/2 - TEXTURE_SIZE; // location on screen
    const int   Y = GAME_HEIGHT/2 - TEXTURE_SIZE;
	
}

// inherits from Entity class
class EntityPlayer : public Entity
{
private:
    float   oldX, oldY, oldAngle;
    float   rotation;                       // current rotation rate (radians/second)
	int		movement;
	int		xMoving;
	int		yMoving;
	byte	standingFrame;
	int		lastDir;
	int		level;
	int		exp;
	int		str;
	int		def;
	int		money;
	int		deaths;
	LARGE_INTEGER lastAttack;
	LARGE_INTEGER tmrFreq;


public:
    // constructor
    EntityPlayer();

    // inherited member functions
    virtual bool initialize(Game *gamePtr, int width, int height, int ncols,
                            TextureManager *textureM);
    void update(float frameTime);
    void repair();

    // new functions
    void setVelocityX(float v)  {velocity.x = v;}

	int		worldX;
	int		worldY;

	int getPlayerX() {return (worldX / 32 * -1) + PLAYER_X_OFFSET;}
	int getPlayerY() {return (worldY / 32 * -1) + PLAYER_Y_OFFSET;}
	void setPlayerX(int x) { worldX = (x - PLAYER_X_OFFSET) * 32 * -1; }
	void setPlayerY(int y) { worldY = (y - PLAYER_Y_OFFSET) * 32 * -1; }
	int getLastDir() { return lastDir; }
	int getXMoving() { return xMoving; }
	int getYMoving() { return yMoving; }
	bool isMoving() { return xMoving != 0 || yMoving != 0; }
	int getMovement() { return movement; }
	int getLevel() { return level; }
	void setLevel(int newLevel);
	void setExp(int newExp);
	void addExp(int addExp);
	int getExp() { return exp; }
	int getNextLevelExp() { return floor(EXP_BASE * pow(EXP_INCREASE, level+1)); }
	void stopMoving();
	int getMaxHealth() { return 100 + (level * 50);  }
	int getPlayerStr();
	int getPlayerBaseStr() { return str + (level * 5) + Sword.metaData; }
	bool EntityPlayer::canAttack();

	int getStrStat() { return str; }
	void setStrStat(int newStr) { str = newStr; }
	int getDefStat() { return def; }
	int getPlayerDef() {return def + Armor.metaData; }
	void setDefStat(int newDef) { def = newDef; }
	int getMoney() { return money; }
	void setMoney(int newMoney) { money = newMoney; }
	int getRespawnCost() { return deaths * 100; }
	ItemInfo Items[PLAYER_MAX_ITEMS];
	ItemInfo Sword;
	ItemInfo Armor;
	ItemInfo Ring;

	

	
	
};
#endif

