#include "HUD.h"


HUD::HUD(void)
{
	statsFont = new TextDX();
	itemInfoFont = new TextDX();
	itemNameFont = new TextDX();
}


HUD::~HUD(void)
{
	safeDelete(statsFont);
	safeDelete(itemInfoFont);
	safeDelete(itemNameFont);
}

void HUD::initialize(Game *theGame) {

	if(hudTexture.initialize(theGame->getGraphics(), HUD_IMAGE) == false)
		throw(GameError(gameErrorNS::FATAL_ERROR, "Error loading HUD"));
	if(armorOverlayTexture.initialize(theGame->getGraphics(), OVERLAY_IMAGE) == false)
		throw(GameError(gameErrorNS::FATAL_ERROR, "Error loading overlays"));
	if(hpTexture.initialize(theGame->getGraphics(), HP_IMAGE) == false)
		throw(GameError(gameErrorNS::FATAL_ERROR, "Error loading hp bar"));
	if(expTexture.initialize(theGame->getGraphics(), EXP_IMAGE) == false)
		throw(GameError(gameErrorNS::FATAL_ERROR, "Error loading exp bar"));
	if (itemTexture.initialize(theGame->getGraphics(),  ITEMS_IMAGE) == false) 
		throw(GameError(gameErrorNS::FATAL_ERROR, "Error loading items textures"));
	if (itemInfoTexture.initialize(theGame->getGraphics(),  ITEM_INFO_IMAGE) == false) 
		throw(GameError(gameErrorNS::FATAL_ERROR, "Error loading item info texture"));


	if (!armorOverlayImage.initialize(theGame->getGraphics(),TEXTURE_SIZE, TEXTURE_SIZE,3,&armorOverlayTexture))
		throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing Armor Overlay"));
	if (!hudImage.initialize(theGame->getGraphics(),0,0,0,&hudTexture))
		throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing HUD"));
	if (!itemImage.initialize(theGame->getGraphics(), TEXTURE_SIZE, TEXTURE_SIZE, 4, &itemTexture))
		throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing item image"));
	if (!hpImage.initialize(theGame->getGraphics(),0,0,0,&hpTexture))
		throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing HP"));
	if (!expImage.initialize(theGame->getGraphics(),0,0,0,&expTexture))
		throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing EXP"));
	if (!itemInfoImage.initialize(theGame->getGraphics(),0,0,0,&itemInfoTexture))
		throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing ItemInfo"));

	if(statsFont->initialize(theGame->getGraphics(), 12, true, false, "Arial") == false)
		throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing stats font"));
	if(itemInfoFont->initialize(theGame->getGraphics(), 12, true, false, "Arial") == false)
		throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing iteminfo font"));
	if(itemNameFont->initialize(theGame->getGraphics(), 16, true, false, "Arial") == false)
		throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing itemname font"));

	hpImage.setX(6+1);
	hpImage.setY(8+1);

	expImage.setX(6+1);
	expImage.setY(30+1);


	armorOverlayImage.setX(378);
}

void HUD::draw(EntityPlayer& thePlayer, int mouseX, int mouseY) {

	hudImage.draw();
	RECT r = hpImage.getSpriteDataRect();
	r.right = (float)hpImage.getWidth() / (float)thePlayer.getMaxHealth() * (float)thePlayer.getHealth();
	if (r.right < 0)
		r.right = 0;
	hpImage.setSpriteDataRect(r);
	hpImage.draw();

	RECT rw = expImage.getSpriteDataRect();
	rw.right = (float)(thePlayer.getExp() / (float)thePlayer.getNextLevelExp()) * (float)expImage.getWidth();
	expImage.setSpriteDataRect(rw);
	expImage.draw();

	//Draw Overlays
	if (thePlayer.Sword.itemID == -1) {
		armorOverlayImage.setCurrentFrame(0);
		armorOverlayImage.setY(653);
		armorOverlayImage.draw();
	}
	if (thePlayer.Armor.itemID == -1) {
		armorOverlayImage.setCurrentFrame(1);
		armorOverlayImage.setY(689);
		armorOverlayImage.draw();
	}
	if (thePlayer.Ring.itemID == -1) {
		armorOverlayImage.setCurrentFrame(2);
		armorOverlayImage.setY(724);
		armorOverlayImage.draw();
	}

	statsFont->print(std::to_string((int)thePlayer.getHealth()) + "/" + std::to_string(thePlayer.getMaxHealth()), hpImage.getX() + 100, hpImage.getY() + 2);
	statsFont->print(std::to_string(thePlayer.getExp()) + "/" + std::to_string(thePlayer.getNextLevelExp()), expImage.getX() + 100, expImage.getY() + 2); 

	statsFont->print(std::to_string(thePlayer.getPlayerBaseStr()), 434, 664);
	statsFont->print(std::to_string(thePlayer.getPlayerDef()), 434, 681);
	statsFont->print(std::to_string(thePlayer.getMoney()), 434, 699);

	for (int i=0; i<PLAYER_MAX_ITEMS; i++) {
		if (thePlayer.Items[i].itemID != -1) {
			itemImage.setX(i % 9 * (TEXTURE_SIZE + 4) + 14);
			itemImage.setY(i / 9 * (TEXTURE_SIZE + 4) + 653);
			itemImage.setCurrentFrame(thePlayer.Items[i].itemTileIndex);
			itemImage.draw();
		}
	}
	if (thePlayer.Sword.itemID != -1) {
		itemImage.setX(378);
		itemImage.setY(653);
		itemImage.setCurrentFrame(thePlayer.Sword.itemTileIndex);
		itemImage.draw();
	}
	if (thePlayer.Armor.itemID != -1) {
		itemImage.setX(378);
		itemImage.setY(689);
		itemImage.setCurrentFrame(thePlayer.Armor.itemTileIndex);
		itemImage.draw();
	}
	if (thePlayer.Ring.itemID != -1) {
		itemImage.setX(378);
		itemImage.setY(724);
		itemImage.setCurrentFrame(thePlayer.Ring.itemTileIndex);
		itemImage.draw();
	}

	
}

void HUD::drawOverlay(EntityPlayer& thePlayer, int mouseX, int mouseY) {
	if (mouseX >= 14 && mouseX <= 333 && mouseY >= 653 && mouseY <= 756) {
		int slot = ((mouseY - 653) / (TEXTURE_SIZE + 4)) * 9 + ((mouseX - 14) / (TEXTURE_SIZE + 4));
		if (thePlayer.Items[slot].itemID != -1) {
			itemInfoImage.setX(mouseX);
			itemInfoImage.setY(mouseY - itemInfoImage.getHeight());
			itemInfoImage.draw();
			itemNameFont->print(thePlayer.Items[slot].ItemName, mouseX+8, mouseY - itemInfoImage.getHeight() + 8);
			itemInfoFont->print(thePlayer.Items[slot].getItemLore(), mouseX+8, mouseY - itemInfoImage.getHeight() + 26);
		}
	}

	if (mouseX >= 378 && mouseX <= 378+32) {
		ItemInfo theItem = ItemInfo();
		if (mouseY >= 653 && mouseY <= 653+32) {
			theItem = thePlayer.Sword;
		} else if (mouseY >= 689 && mouseY <= 689+32) {
			theItem = thePlayer.Armor;
		} else if (mouseY >= 724 && mouseY <= 724+32) {
			theItem = thePlayer.Ring;
		}
		if (theItem.itemID != -1) {
			itemInfoImage.setX(mouseX);
			itemInfoImage.setY(mouseY - itemInfoImage.getHeight());
			itemInfoImage.draw();
			itemNameFont->print(theItem.ItemName, mouseX+8, mouseY - itemInfoImage.getHeight() + 8);
			itemInfoFont->print(theItem.getItemLore(), mouseX+8, mouseY - itemInfoImage.getHeight() + 26);
		}
	}
}

void HUD::releaseAll()
{
	statsFont->onLostDevice();
	itemInfoFont->onLostDevice();
	itemNameFont->onLostDevice();
}

void HUD::resetAll()
{
	statsFont->onResetDevice();
	itemInfoFont->onResetDevice();
	itemNameFont->onResetDevice();
}
