#ifndef _ITEMINFO_H            // Prevent multiple definitions if this 
#define _ITEMINFO_H            // file is included in more than one place
#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include "constants.h"
#include <string>

//1 = Sword
//2 = Sheild
//3 = 
//4 = Steak
//5 = Small Potion - 10%
//6 = Medium Potion - 25%
//7 = Large Potion - 50%
//8 = Gigantic Potion - 100%
//9 = Stairs Scroll
//10 = Teleport Scroll
//11 = Money
//12 = Healing Ring
//13 = Speed Ring
//14 = Str increase book
//15 = Def increase book
//16 = Reduced Death Cost Ring
//17 = Pickaxe - Super rare - Does not spawn if player already owns one
//18 = Map Book
//19 = 10% Exp Boost Ring


class ItemInfo
{
private:
	void findReplace(std::string& str, const std::string& oldStr, const std::string& newStr);
public:
	std::string ItemName;
	std::string ItemLore;
	std::string ItemLore2;
	int itemID;			//Must be unique
	int itemTileIndex;
	int metaData;
	ITEM_SLOT type;

	//Level and player use
	int xCoord;
	int yCoord;
	
	ItemInfo() {xCoord = 0; yCoord = 0; itemID = -1; metaData = 0;}
	ItemInfo(std::string theName, std::string theLore, int id, int tileIndex, ITEM_SLOT itemType);
	std::string getItemLore();
};




const ItemInfo SWORD("Sword", "Equipable in your sword slot\nAttack: %meta%", 0, 8, WEAPON);
const ItemInfo SHIELD("Armor", "Equipable in your defence slot\nDefence: %meta%", 1, 12, ARMOR);
const ItemInfo FOOD("Steak", "This is not pickupable", 2, 18, NORMAL);
const ItemInfo SMALL_POTION("Small Health Potion", "Heals 10% of your health", 3, 13, NORMAL);
const ItemInfo MEDIUM_POTION("Medium Health Potion", "Heals 25% of your health", 4, 17, NORMAL);
const ItemInfo LARGE_POTION("Large Health Potion", "Heals 50% of your health", 5, 21, NORMAL);
const ItemInfo GIGANTIC_POTION("Gigantic Health Potion", "Fully heals the user's health", 6, 9, NORMAL);
const ItemInfo STAIRS_TELEPORT_SCROLL("Exit Teleport Scroll", "Teleports the user to the floors exit", 7, 4, NORMAL);
const ItemInfo TELEPORT_SCROLL("Teleport Scroll", "Teleports the user to a random location in the\nfloor. Useful for evading monsters.", 8, 5, NORMAL);
const ItemInfo MONEY_BAG("Money", "This is not pickupable", 9, 0, NORMAL);
const ItemInfo HEALING_RING("Healing Ring", "Heals the user over time", 10, 7, RING);
const ItemInfo SPEED_RING("Speed Ring", "Makes the user twice as fast", 11, 11, RING);
const ItemInfo DEATH_RING("Angel Ring", "Reduces users cost of continuing upon death", 12, 3, RING);
const ItemInfo EXP_RING("Exp Ring", "The user gains 10% more exp per kill", 13, 15, RING);
const ItemInfo STR_BOOK("Strength Book", "Increases the users strength", 14, 1, NORMAL);
const ItemInfo DEF_BOOK("Defence Book", "Increases the users defence", 15, 2, NORMAL);
const ItemInfo PICKAXE("Pickaxe", "Used to break down stable one thick walls", 16, 6, NORMAL);
const ItemInfo MAP_BOOK("Map", "Reveals the entire floor", 17, 10, NORMAL);



static ItemInfo itemList[] = {SWORD, SHIELD, FOOD, SMALL_POTION, MEDIUM_POTION, LARGE_POTION, GIGANTIC_POTION, STAIRS_TELEPORT_SCROLL, TELEPORT_SCROLL, MONEY_BAG, HEALING_RING, SPEED_RING, DEATH_RING, EXP_RING, STR_BOOK,
	DEF_BOOK, PICKAXE, MAP_BOOK};


#endif