// Programming 2D Games
// Copyright (c) 2011 by: 
// Charles Kelly
// createThisClass.cpp v1.1
// This class is the core of the game

#include "createThisClass.h"

//=============================================================================
// Constructor
//=============================================================================
CreateThis::CreateThis()
{
	dxFont = new TextDX();  // DirectX font
	theLevel = new levelManager(thePlayer, *dxFont);
}

//=============================================================================
// Destructor
//=============================================================================
CreateThis::~CreateThis() 
{
	releaseAll();           // call onLostDevice() for every graphics item
	safeDelete(dxFont);
}

//=============================================================================
// Initializes the game
// Throws GameError on error
//=============================================================================
void CreateThis::initialize(HWND hwnd)
{
	Game::initialize(hwnd); // throws GameError

	// Init Tile Sheet
	if(spriteSheet.initialize(graphics, MENU_IMAGE) == false)
		throw(GameError(gameErrorNS::FATAL_ERROR, "Error loading tile sheet"));


	if (tile.initialize(graphics, TEXTURE_SIZE, TEXTURE_SIZE, TEXTURE_SHEET_COLS, &spriteSheet) == false) {
		throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing tile"));
	}
	if (playerSprites.initialize(graphics,  PLAYER_IMAGE) == false) {
		throw(GameError(gameErrorNS::FATAL_ERROR, "Error loading player sprite sheet"));
	}
	if (thePlayer.initialize(this, TEXTURE_SIZE, TEXTURE_SIZE, 4, &playerSprites) == false) {
		throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing player"));
	}

	thePlayer.setFrames(0,0);
	thePlayer.setCurrentFrame(0);
	thePlayer.setHealth(100);


	theLevel->initialize(this);
	myHud.initialize(this);

	// initialize DirectX font
	// 18 pixel high Arial
	if(dxFont->initialize(graphics, chatFontSize, true, false, "Arial") == false)
		throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing DirectX font"));

	theLevel->preGenNext();
	floorGenerator.generateNewFloor(*theLevel, thePlayer.getLastDir(), false);
	floorGenerator.generateItems(*theLevel, itemManager, thePlayer);
	theLevel->postGenNext();

	needToClearFog = true;

	infochat::init();	

	noClip = false;
	stopMobs = false;

	audio->playCue(MUSIC);


	return;
}

//=============================================================================
// Update all game items
//=============================================================================
void CreateThis::update()
{

	thePlayer.update(frameTime);

	if (thePlayer.getXMoving() == MOVING_LENGTH || thePlayer.getYMoving() ==  MOVING_LENGTH) {
		needToClearFog = true;
	} else if (thePlayer.isMoving() == false && needToClearFog) {
		theLevel->clearFog();
		needToClearFog = false;
		//FIND ROOM
		int xCoordLeft = 0;
		int xCoordRight = 0;
		int yCoordUp = 0;
		int yCoordDown = 0;
		for (int x = thePlayer.getPlayerX(); x>0; x--) {
			if (theLevel->getTileAt(0, x, thePlayer.getPlayerY()) >= 0) {
				//Is a door?
				if (theLevel->isWallAt(x, thePlayer.getPlayerY()-1) && theLevel->isWallAt(x, thePlayer.getPlayerY()+1)) {
					xCoordLeft = x+1;
					break;
				}
			} else {
				xCoordLeft = x+1;
				break;
			}
		}
		for (int x = thePlayer.getPlayerX(); x<thePlayer.getPlayerX() + 100; x++) {
			if (theLevel->getTileAt(0, x, thePlayer.getPlayerY()) >= 0) {
				//Is a door?
				if (theLevel->isWallAt(x, thePlayer.getPlayerY()-1) && theLevel->isWallAt(x, thePlayer.getPlayerY()+1)) {
					xCoordRight = x-1;
					break;
				}
			} else {
				xCoordRight = x-1;
				break;
			}
		}
		for (int y = thePlayer.getPlayerY(); y>0; y--) {
			if (theLevel->getTileAt(0, thePlayer.getPlayerX(), y) >= 0) {
				//Is a door?
				if (theLevel->isWallAt(thePlayer.getPlayerX() - 1, y) && theLevel->isWallAt(thePlayer.getPlayerX() + 1, y)) {
					yCoordUp = y+1;
					break;
				}
			} else {
				yCoordUp = y+1;
				break;
			}
		}
		for (int y = thePlayer.getPlayerY(); y<thePlayer.getPlayerY() + 100; y++) {
			if (theLevel->getTileAt(0, thePlayer.getPlayerX(), y) >= 0) {
				//Is a door?
				if (theLevel->isWallAt(thePlayer.getPlayerX() - 1, y) && theLevel->isWallAt(thePlayer.getPlayerX() + 1, y)) {
					yCoordDown = y-1;
					break;
				}
			} else {
				yCoordDown = y-1;
				break;
			}
		}

		for (int i=xCoordLeft-1; i<=xCoordRight+1; i++) {
			for (int a=yCoordUp-1; a<=yCoordDown+1; a++) {
				//theLevel->setTileAt(ABOVE_PLAYER_LAYER, i, a, 0);
				theLevel->setFogTileAt(i,a,-1);
			}
		}

		//END FIND ROOM
	}

	if (input->wasKeyPressed(ENTER_KEY)) {
		itemManager.handleItemUse(thePlayer, *theLevel, theLevel->checkItemCollision());
	}

	if (input->wasKeyPressed(CONSOLE_KEY)) {
		console->showHide();		
	}
	static bool waitForMouse = false;
	if (input->getMouseLButton()) {
		if (!waitForMouse) {
			waitForMouse = true;
			if (input->getMouseX() >= 14 && input->getMouseX() <= 333 && input->getMouseY() >= 653 && input->getMouseY() <= 756) {
				int slot = ((input->getMouseY() - 653) / (TEXTURE_SIZE + 4)) * 9 + ((input->getMouseX() - 14) / (TEXTURE_SIZE + 4));
				itemManager.handleItemClick(thePlayer, *theLevel, thePlayer.Items[slot], slot);
			}
		}
	} else if (input->getMouseMButton()) {
		if (!waitForMouse) {
			waitForMouse = true;
			if (input->getMouseX() >= 14 && input->getMouseX() <= 333 && input->getMouseY() >= 653 && input->getMouseY() <= 756) {
				int slot = ((input->getMouseY() - 653) / (TEXTURE_SIZE + 4)) * 9 + ((input->getMouseX() - 14) / (TEXTURE_SIZE + 4));
				thePlayer.Items[slot] = ItemInfo();
			}
		}
	} else {
		waitForMouse = false;
	}

	theLevel->updateMobs(frameTime);

}

void CreateThis::consoleCommand()
{
	command = console->getCommand();    // get command from console
	if(command == "")                   // if no command
		return;

	if (command == "help")              // if "help" command
	{
		console->print("Console Commands:");
		console->print("fps - toggle display of frames per second");
		console->print("fillInv - fill players inventory with random items");
		console->print("fillInvO - fill players inventory with random items overwriting anything already there");
		console->print("noclip - Toggles noclip");
		console->print("stopmobs - Toggles mob activity. Does not stop spawning.");
		return;
	}

	if (command == "fps")
	{
		fpsOn = !fpsOn;                 // toggle display of fps
		if(fpsOn)
			console->print("fps On");
		else
			console->print("fps Off");
	} else if (command == "fillInv") {
		int rnd;
		for (int i=0; i<PLAYER_MAX_ITEMS; i++) {
reLoop:
			rnd = rand() % RND_MAX_ITEMS;
			thePlayer.Items[i].itemID = itemList[itemManager.itemPool[rnd]].itemID;
			thePlayer.Items[i].ItemLore = itemList[itemManager.itemPool[rnd]].ItemLore;
			thePlayer.Items[i].ItemName = itemList[itemManager.itemPool[rnd]].ItemName;
			thePlayer.Items[i].itemTileIndex = itemList[itemManager.itemPool[rnd]].itemTileIndex;
			thePlayer.Items[i].type =  itemList[itemManager.itemPool[rnd]].type;
			if (thePlayer.Items[i].itemID == SWORD.itemID || thePlayer.Items[i].itemID == SHIELD.itemID) {
				thePlayer.Items[i].metaData = theLevel->level * 10 + (rand() % 10) + 1;
			}
			if (thePlayer.Items[i].itemID == FOOD.itemID || thePlayer.Items[i].itemID == MONEY_BAG.itemID) {
				goto reLoop;
			}
			console->print("Adding item " + thePlayer.Items[i].ItemName + " (" + thePlayer.Items[i].ItemLore + ") Meta: " + std::to_string(thePlayer.Items[i].metaData) + " to slot " + std::to_string(i));			
		}
	} else if (command == "noclip") {
		noClip = !noClip;
		console->print("NoClip toggled to " + std::to_string(noClip));
	}else if (command == "pickaxe") {
		thePlayer.Items[0] = PICKAXE;
		console->print("Pickaxe Spawned");
	} else if (command == "stopmobs") {
		stopMobs = !stopMobs;
		console->print("StopMobs toggled to " + std::to_string(stopMobs));
	} else if (command == "hurt") {
		thePlayer.setHealth(10);
		console->print("Hurt the player");
	}else {
		console->print("Bad command: " + command);
	}
}

//=============================================================================
// Artificial Intelligence
//=============================================================================
void CreateThis::ai()
{
	static bool doRun = false;
	if (theLevel->runMobAI) {
		theLevel->runMobAI = false;
		if (thePlayer.Ring.itemID == SPEED_RING.itemID) {
			if (doRun) {
				doRun = false;
				theLevel->aiMobs(frameTime, true);
			} else {
				doRun = true;
				theLevel->aiMobs(frameTime, false);
			}
		} else {
			theLevel->aiMobs(frameTime, true);
		}
	}
	if (thePlayer.getHealth() <= 0) {
		int costToRespawn = thePlayer.getRespawnCost() + ((theLevel->level % 4) * 100);
		costToRespawn = thePlayer.Ring.itemID == DEATH_RING.itemID ? costToRespawn / 2 : costToRespawn;
		if (thePlayer.getMoney() >= costToRespawn) {
			infochat::addChat("Oh no you have passed out!");
			infochat::addChat("You have awoken in another area, you seem to have lost " + std::to_string(costToRespawn) + " gold.");
			thePlayer.setMoney(thePlayer.getMoney() - costToRespawn);
			Coords newPlace = theLevel->getSpawnableCoords();
			if (newPlace.xCoord != 0 || newPlace.yCoord != 0) {
				thePlayer.setPlayerX(newPlace.xCoord);
				thePlayer.setPlayerY(newPlace.yCoord);
				thePlayer.getAudio()->playCue(teleport);				
			}
			theLevel->clearFog();
		} else {
			infochat::reinit();
			infochat::addChat("Oh no you have passed out!");
			infochat::addChat("Game Over.");
			infochat::addChat("Previous Stats:");
			infochat::addChat("    Floor: " + std::to_string(theLevel->level));
			infochat::addChat("    Level: " + std::to_string(thePlayer.getLevel()));
			infochat::addChat("    Attack: " + std::to_string(thePlayer.getStrStat()));
			infochat::addChat("    Defence: " + std::to_string(thePlayer.getDefStat()));
			infochat::addChat("    Money: " + std::to_string(thePlayer.getMoney()));
			infochat::addChat("Welcome to Dungeon Crawler");
			thePlayer.setMoney(0);
			thePlayer.setStrStat(PLAYER_BASE_STR);
			thePlayer.setLevel(0);
			thePlayer.setExp(0);
			thePlayer.setDefStat(0);
			thePlayer.setHealth(thePlayer.getMaxHealth());
			thePlayer.Sword = ItemInfo();
			thePlayer.Ring = ItemInfo();
			thePlayer.Armor = ItemInfo();
			for (int i=0; i<PLAYER_MAX_ITEMS; i++) {
				thePlayer.Items[i] = ItemInfo();
			}
			theLevel->level=-1;
			theLevel->preGenNext();
			floorGenerator.generateNewFloor(*theLevel, thePlayer.getLastDir(), false);
			floorGenerator.generateItems(*theLevel, itemManager, thePlayer);
			theLevel->postGenNext();
			needToClearFog = true;
		}

	}

	//theLevel->aiMobs(frameTime);
}

//=============================================================================
// Handle collisions
//=============================================================================
void CreateThis::collisions()
{
	if (thePlayer.getPlayerX() == theLevel->stairs.xCoord && thePlayer.getPlayerY() == theLevel->stairs.yCoord) {
		theLevel->preGenNext();
		floorGenerator.generateNewFloor(*theLevel, thePlayer.getLastDir(), false);
		floorGenerator.generateItems(*theLevel, itemManager, thePlayer);
		theLevel->postGenNext();
		needToClearFog = true;
	}

	if (thePlayer.getPlayerX() == theLevel->stairsUp.xCoord && thePlayer.getPlayerY() == theLevel->stairsUp.yCoord) {
		theLevel->preGenPrev();
		floorGenerator.generateNewFloor(*theLevel, thePlayer.getLastDir(), true);
		theLevel->postGenPrev();
		needToClearFog = true;
	}

	if (thePlayer.getXMoving() == MOVING_LENGTH || thePlayer.getYMoving() ==  MOVING_LENGTH) {
		if (noClip) {
			theLevel->runMobAI = !stopMobs;
			return;
		}
		switch(thePlayer.getLastDir()) {
		case DIR_LEFT:
			if (!theLevel->isTileUnobtrustive(thePlayer.getPlayerX()-1, thePlayer.getPlayerY(), -1)) {
				thePlayer.stopMoving();
				EntityMonster *currentTarget = theLevel->getEntityMonsterAt(thePlayer.getPlayerX()-1, thePlayer.getPlayerY());
				if (currentTarget != NULL)
					theLevel->runMobAI = theLevel->playerAttackMonster(*currentTarget);
			} else {
				theLevel->runMobAI = true;
			}
			break;

		case DIR_UP:
			if (!theLevel->isTileUnobtrustive(thePlayer.getPlayerX(), thePlayer.getPlayerY()-1, -1))  {
				thePlayer.stopMoving();
				EntityMonster *currentTarget = theLevel->getEntityMonsterAt(thePlayer.getPlayerX(), thePlayer.getPlayerY()-1);
				if (currentTarget != NULL)
					theLevel->runMobAI = theLevel->playerAttackMonster(*currentTarget);
			} else {
				theLevel->runMobAI = true;
			}
			break;

		case DIR_RIGHT:
			if (!theLevel->isTileUnobtrustive(thePlayer.getPlayerX()+1, thePlayer.getPlayerY(), -1)) {
				thePlayer.stopMoving();
				EntityMonster *currentTarget = theLevel->getEntityMonsterAt(thePlayer.getPlayerX()+1, thePlayer.getPlayerY());
				if (currentTarget != NULL)
					theLevel->runMobAI = theLevel->playerAttackMonster(*currentTarget);
			} else {
				theLevel->runMobAI = true;
			}
			break;

		case DIR_DOWN:
			if (!theLevel->isTileUnobtrustive(thePlayer.getPlayerX(), thePlayer.getPlayerY()+1, -1))  {
				thePlayer.stopMoving();
				EntityMonster *currentTarget = theLevel->getEntityMonsterAt(thePlayer.getPlayerX(), thePlayer.getPlayerY()+1);
				if (currentTarget != NULL)
					theLevel->runMobAI = theLevel->playerAttackMonster(*currentTarget);
			} else {
				theLevel->runMobAI = true;
			}
			break;
		}

		if (stopMobs) {
			theLevel->runMobAI = false;
		}
	}
}

//=============================================================================
// Render game items
//=============================================================================
void CreateThis::render()
{
	graphics->spriteBegin();                // begin drawing sprites

	int tileX = thePlayer.worldX / 32 * -1;
	int tileY = thePlayer.worldY / 32 * -1;

	//Go one level below so that you cannot see the world "generating" after they go past -1/-1; getTileAt will safely ignore the invalid requests.
	for (int x = tileX-1; x < tileX + SCREEN_WIDTH; x++) {
		tile.setX(x*TEXTURE_SIZE + thePlayer.worldX);
		for (int y = tileY-1; y < tileY + SCREEN_HEIGHT; y++) {
			tile.setY(y * TEXTURE_SIZE + thePlayer.worldY);
			srand(x * y);
			tile.setCurrentFrame(backgroundTiles[rand() % backgroundTilesCount]);
			tile.draw();

			for (int level=0; level<LEVEL_LAYERS-1; level++) {
				if (theLevel->getTileAt(level,x,y) > -1) {
					tile.setCurrentFrame(theLevel->getTileAt(level,x,y));
					tile.draw();
				}
			}


		}
	}
	srand(time(NULL));
	theLevel->drawItems();

	thePlayer.draw();
	theLevel->drawMobs();


	//Render a cosmetic layer above the player.
	for (int x = tileX-1; x < tileX + SCREEN_WIDTH; x++) {
		tile.setX(x*TEXTURE_SIZE + thePlayer.worldX);
		for (int y = tileY-1; y < tileY + SCREEN_HEIGHT; y++) {
			if (theLevel->getTileAt(ABOVE_PLAYER_LAYER,x,y) > -1) {
				tile.setY(y * TEXTURE_SIZE + thePlayer.worldY);
				tile.setCurrentFrame(theLevel->getTileAt(ABOVE_PLAYER_LAYER,x,y));
				tile.draw();
			}
		}
	}

	//Render a fog layer. This is cleared automaticlly as the player moves around the level.
	for (int x = tileX-1; x < tileX + SCREEN_WIDTH; x++) {
		tile.setX(x*TEXTURE_SIZE + thePlayer.worldX);
		for (int y = tileY-1; y < tileY + SCREEN_HEIGHT; y++) {
			if (theLevel->getFogTileAt(x,y) > -1) {
				tile.setY(y * TEXTURE_SIZE + thePlayer.worldY);
				tile.setCurrentFrame(theLevel->getFogTileAt(x,y));
				tile.draw();
			}
		}
	}

	//HUD

	dxFont->setFontColor(graphicsNS::ORANGE);
	dxFont->print("WorldX: " + std::to_string(thePlayer.worldX) + " - TileX: " + std::to_string(tileX) + " - LevelX: " + std::to_string(theLevel->minX) + "/" + std::to_string(theLevel->maxX) + " - PlayerX: " + std::to_string(thePlayer.getPlayerX()), 20, 60);
	dxFont->print("WorldY: " + std::to_string(thePlayer.worldY) + " - TileY: " + std::to_string(tileY) + " - LevelY: " + std::to_string(theLevel->minY) + "/" + std::to_string(theLevel->maxY) + " - PlayerY: " + std::to_string(thePlayer.getPlayerY()), 20, 80);
	dxFont->print("Level: " + std::to_string(theLevel->level), 20, 100);
	dxFont->print("EXP: " + std::to_string(thePlayer.getExp()) + " - Level: " + std::to_string(thePlayer.getLevel()), 20, 120);




	myHud.draw(thePlayer, input->getMouseX(), input->getMouseY());
	infochat::drawChat(dxFont);
	myHud.drawOverlay(thePlayer, input->getMouseX(), input->getMouseY());

	//Draw Player Items

	console->draw();

	graphics->spriteEnd();                  // end drawing sprites


}


//=============================================================================
// The graphics device was lost.
// Release all reserved video memory so graphics device may be reset.
//=============================================================================
void CreateThis::releaseAll()
{
	dxFont->onLostDevice();
	theLevel->nameFont->onLostDevice();
	spriteSheet.onLostDevice();
	Game::releaseAll();
	myHud.releaseAll();
	return;
}

//=============================================================================
// The grahics device has been reset.
// Recreate all surfaces.
//=============================================================================
void CreateThis::resetAll()
{
	//menuTexture.onResetDevice();
	spriteSheet.onResetDevice();
	dxFont->onResetDevice();
	theLevel->nameFont->onResetDevice();
	Game::resetAll();
	myHud.resetAll();
	return;
}
