// Blake Price
// floorGenerator.cpp
// Generates a new dungeon floor
#include "floorGenerator.h"



//=============================================================================
// Constructor
//=============================================================================
floorGen::floorGen()
{

}

//=============================================================================
// Destructor
//=============================================================================
floorGen::~floorGen()
{

}

// Generate a new dungeon floor
void floorGen::generateNewFloor(levelManager& theLevel, int playerLastDirection, bool goingUp) 
{


	int anX = 15;
	int anY = 15;

	int roomWidth = ROOM_MIN_SIZE + rand() % ROOM_MAX_SIZE;
	int roomHeight = ROOM_MIN_SIZE + rand() % ROOM_MAX_SIZE;
	theLevel.roomCount = 0;

	for (int l=0; l<LEVEL_LAYERS; l++) {
		for (int x=0; x<LEVEL_WIDTH; x++) {
			for (int y=0; y<LEVEL_HEIGHT; y++) {
				theLevel.setTileAt(l,x,y, -1);
			}
		}
	}
	for (int x=0; x<LEVEL_WIDTH; x++) {
		for (int y=0; y<LEVEL_HEIGHT; y++) {
			srand (x*y);
			theLevel.setFogTileAt(x,y,backgroundTiles[rand() % backgroundTilesCount]);
		}
	}
	srand (time(NULL));
	generateRoom(anX, anY, roomWidth, roomHeight, theLevel, 0);

	generateStairs(theLevel, false);

	if (theLevel.level > 0)
		generateStairs(theLevel, true);

	spawnPlayerCoords(theLevel, playerLastDirection, goingUp);






}
void floorGen::generateItems(levelManager& theLevel, ItemManager& itemManager, EntityPlayer& thePlayer) {
	/*itemsOnFloor = roomCount * 2;
	if (itemsOnFloor < MAX_ITEMS-1)
	itemsOnFloor += rand() % (MAX_ITEMS - itemsOnFloor);
	if (itemsOnFloor >= MAX_ITEMS)
	itemsOnFloor = MAX_ITEMS-1;*/
	theLevel.itemsOnFloor = ITEMS_SPAWNED_PER_FLOOR;

	Coords itemCoord;

	//DEBUG
	char  buffer[200];
	sprintf(buffer, "Spawning %d items\n", theLevel.itemsOnFloor);
	OutputDebugString(buffer);

	int rnd;
	for (int i=0; i<theLevel.itemsOnFloor; i++) {
		rnd = rand() % RND_MAX_ITEMS;
		itemCoord = theLevel.getItemSpawnableCoords();
		theLevel.Items[i].itemID = itemList[itemManager.itemPool[rnd]].itemID;
		theLevel.Items[i].ItemLore = itemList[itemManager.itemPool[rnd]].ItemLore;
		theLevel.Items[i].ItemName = itemList[itemManager.itemPool[rnd]].ItemName;
		theLevel.Items[i].itemTileIndex = itemList[itemManager.itemPool[rnd]].itemTileIndex;
		theLevel.Items[i].type =  itemList[itemManager.itemPool[rnd]].type;
		theLevel.Items[i].xCoord = itemCoord.xCoord;
		theLevel.Items[i].yCoord = itemCoord.yCoord;
		if (theLevel.Items[i].itemID == SWORD.itemID || theLevel.Items[i].itemID == SHIELD.itemID) {
			theLevel.Items[i].metaData = theLevel.level * 10 + (rand() % 10) + 1;
		}
		//Items[i].generateRandomItem(level);
		sprintf(buffer, "Spawning %s at %d:%d\n", theLevel.Items[i].ItemName.c_str(), itemCoord.xCoord,itemCoord.yCoord);
		OutputDebugString(buffer);
	}
	for (int i=theLevel.itemsOnFloor; i<MAX_ITEMS; i++) {
		theLevel.Items[i].xCoord = -100;
		theLevel.Items[i].yCoord = -100;
	}
}

void floorGen::spawnPlayerCoords(levelManager& theLevel, int playerLastDirection, bool goingUp) {
	if (theLevel.level > 0 && !goingUp) {
		switch (playerLastDirection) {
		case DIR_LEFT:
			theLevel.spawn.xCoord = theLevel.stairsUp.xCoord-1;
			theLevel.spawn.yCoord = theLevel.stairsUp.yCoord;
			break;

		case DIR_UP:
			theLevel.spawn.xCoord = theLevel.stairsUp.xCoord;
			theLevel.spawn.yCoord = theLevel.stairsUp.yCoord-1;
			break;

		case DIR_RIGHT:
			theLevel.spawn.xCoord = theLevel.stairsUp.xCoord+1;
			theLevel.spawn.yCoord = theLevel.stairsUp.yCoord;
			break;

		case DIR_DOWN:
			theLevel.spawn.xCoord = theLevel.stairsUp.xCoord;
			theLevel.spawn.yCoord = theLevel.stairsUp.yCoord+1;
			break;
		}
	} else if (goingUp) {
		switch (playerLastDirection) {
		case DIR_LEFT:
			theLevel.spawn.xCoord = theLevel.stairs.xCoord-1;
			theLevel.spawn.yCoord = theLevel.stairs.yCoord;
			break;

		case DIR_UP:
			theLevel.spawn.xCoord = theLevel.stairs.xCoord;
			theLevel.spawn.yCoord = theLevel.stairs.yCoord-1;
			break;

		case DIR_RIGHT:
			theLevel.spawn.xCoord = theLevel.stairs.xCoord+1;
			theLevel.spawn.yCoord = theLevel.stairs.yCoord;
			break;

		case DIR_DOWN:
			theLevel.spawn.xCoord = theLevel.stairs.xCoord;
			theLevel.spawn.yCoord = theLevel.stairs.yCoord+1;
			break;
		}
	} else {
		theLevel.spawn = theLevel.getSpawnableCoords();
	}
}

void floorGen::generateStairs(levelManager& theLevel, bool up) {

	if (up) {
		theLevel.stairsUp = theLevel.getSpawnableCoords();
		theLevel.setTileAt(1,theLevel.stairsUp.xCoord, theLevel.stairsUp.yCoord, stairsUp[rand() % 2]);
	} else {
		theLevel.stairs = theLevel.getSpawnableCoords();
		theLevel.setTileAt(1,theLevel.stairs.xCoord, theLevel.stairs.yCoord, stairsDown[rand() % 2]);
	}
	char  buffer[200];
	sprintf(buffer, "Put Stairs At: %d:%d\n", up ? theLevel.stairsUp.xCoord : theLevel.stairs.xCoord, up ? theLevel.stairsUp.yCoord : theLevel.stairs.yCoord);
	OutputDebugString(buffer);
}

bool floorGen::generateRoom(int anX, int anY, int roomWidth, int roomHeight, levelManager& theLevel, int recursLevel) {
	if (recursLevel < 8 && doesRoomFit(anX, anY, roomWidth, roomHeight, theLevel)) {
		char  buffer[200];
		sprintf(buffer, "Putting room at: %d:%d\n", anX, anY);
		OutputDebugString(buffer);
		insertRoom(anX, anY, roomWidth, roomHeight, theLevel);

		theLevel.setMinY(anY);
		theLevel.setMaxY(anY + roomHeight);
		theLevel.setMinX(anX);
		theLevel.setMaxX(anX + roomWidth);

		int newRoomX;
		int newRoomY;
		int newRoomWidth;
		int newRoomHeight;
		int doorX;
		int doorY;
		for (int i=2; i<4; i++) {
			if (rand() % CHANCE_TO_GEN_ROOM <= CHANCE_LESS_THAN) {
				newRoomWidth = ROOM_MIN_SIZE + rand() % ROOM_MAX_SIZE;
				newRoomHeight = ROOM_MIN_SIZE + rand() % ROOM_MAX_SIZE;
				for (int a=0; a<ATTEMPTS_BEFORE_FAIL; a++) {
					switch (i) {
					case DIR_LEFT:
						doorX = anX-1;
						doorY = anY + rand() % roomHeight;
						newRoomX = anX - newRoomWidth - 1;
						newRoomY = doorY - rand() % newRoomHeight;
						break;
					case DIR_UP:
						doorX = anX + rand() % roomWidth;
						doorY = anY-1;
						newRoomX = doorX - rand() % newRoomWidth;
						newRoomY = anY - newRoomHeight - 1;
						break;
					case DIR_RIGHT:
						doorX = anX + roomWidth;
						doorY = anY + rand() % roomHeight;
						newRoomX = anX + roomWidth + 1;
						newRoomY = doorY - rand() % newRoomHeight;
						break;
					case DIR_DOWN:
						doorX = anX + rand() % roomWidth;
						doorY = anY + roomHeight;
						newRoomX = doorX - rand() % newRoomWidth;
						newRoomY = anY + roomHeight + 1;
						break;
					}
					if (generateRoom(newRoomX, newRoomY, newRoomWidth, newRoomHeight, theLevel, recursLevel+1)) {
						if (i == DIR_LEFT || i == DIR_RIGHT) {
							if ((theLevel.getTileAt(4,doorX + 1,doorY - 1) == topWall || theLevel.getTileAt(4,doorX + 1,doorY - 1) == doubleWallVertical) && (theLevel.getTileAt(4,doorX - 1,doorY - 1) == topWall || theLevel.getTileAt(4,doorX - 1,doorY - 1) == doubleWallVertical))
								theLevel.setTileAt(4,doorX,doorY - 1, topWall);
							else if (theLevel.getTileAt(4,doorX + 1,doorY - 1) == topWall || theLevel.getTileAt(4,doorX + 1,doorY - 1) == doubleWallVertical)
								theLevel.setTileAt(4,doorX,doorY - 1, cornerSouthWest);
							else if (theLevel.getTileAt(4,doorX - 1,doorY - 1) == topWall || theLevel.getTileAt(4,doorX - 1,doorY - 1) == doubleWallVertical)
								theLevel.setTileAt(4,doorX,doorY - 1, cornerSouthEast);
							else
								theLevel.setTileAt(4,doorX,doorY - 1, verticalDoorTop);

							if ((theLevel.getTileAt(4,doorX + 1,doorY + 1) == bottomWall || theLevel.getTileAt(4,doorX + 1,doorY + 1) == doubleWallVertical) && (theLevel.getTileAt(4,doorX - 1,doorY + 1) == bottomWall || theLevel.getTileAt(4,doorX - 1,doorY + 1) == doubleWallVertical))
								theLevel.setTileAt(4,doorX,doorY + 1, bottomWall);
							else if (theLevel.getTileAt(4,doorX + 1,doorY + 1) == bottomWall || theLevel.getTileAt(4,doorX + 1,doorY + 1) == doubleWallVertical)
								theLevel.setTileAt(4,doorX,doorY + 1, cornerNorthWest);
							else if (theLevel.getTileAt(4,doorX - 1,doorY + 1) == bottomWall || theLevel.getTileAt(4,doorX - 1,doorY + 1) == doubleWallVertical)
								theLevel.setTileAt(4,doorX,doorY + 1, cornerNorthEast);
							else
								theLevel.setTileAt(4,doorX,doorY + 1, verticalDoorBottom);

						} else {
							if ((theLevel.getTileAt(4,doorX - 1,doorY + 1) == leftWall || theLevel.getTileAt(4,doorX - 1,doorY + 1) == doubleWallHorizontal) && (theLevel.getTileAt(4,doorX - 1,doorY - 1) == leftWall || theLevel.getTileAt(4,doorX - 1,doorY - 1) == doubleWallHorizontal))
								theLevel.setTileAt(4,doorX - 1,doorY, leftWall);
							else if (theLevel.getTileAt(4,doorX - 1,doorY + 1) == leftWall || theLevel.getTileAt(4,doorX - 1,doorY + 1) == doubleWallHorizontal)
								theLevel.setTileAt(4,doorX - 1,doorY, cornerNorthEast);
							else if (theLevel.getTileAt(4,doorX - 1,doorY - 1) == leftWall || theLevel.getTileAt(4,doorX - 1,doorY - 1) == doubleWallHorizontal)
								theLevel.setTileAt(4,doorX - 1,doorY, cornerSouthEast);
							else if (theLevel.getTileAt(4, doorX - 1, doorY) != verticalDoorBottom && theLevel.getTileAt(4, doorX - 1, doorY) != verticalDoorTop)
								theLevel.setTileAt(4,doorX - 1,doorY, horizontalDoorLeft);


							if ((theLevel.getTileAt(4,doorX + 1,doorY - 1) == rightWall || theLevel.getTileAt(4,doorX + 1,doorY - 1) == doubleWallHorizontal) && (theLevel.getTileAt(4,doorX + 1,doorY + 1) == rightWall || theLevel.getTileAt(4,doorX + 1,doorY + 1) == doubleWallHorizontal))
								theLevel.setTileAt(4,doorX + 1,doorY, rightWall);
							else if (theLevel.getTileAt(4,doorX + 1,doorY - 1) == rightWall || theLevel.getTileAt(4,doorX + 1,doorY - 1) == doubleWallHorizontal)
								theLevel.setTileAt(4,doorX + 1,doorY, cornerSouthWest);
							else if (theLevel.getTileAt(4,doorX + 1,doorY + 1) == rightWall || theLevel.getTileAt(4,doorX + 1,doorY + 1) == doubleWallHorizontal)
								theLevel.setTileAt(4,doorX + 1,doorY, cornerNorthWest);
							else if (theLevel.getTileAt(4,doorX + 1,doorY) != verticalDoorBottom && theLevel.getTileAt(4,doorX + 1,doorY) != verticalDoorTop)
								theLevel.setTileAt(4,doorX + 1,doorY, horizontalDoorRight);


						}
						theLevel.setTileAt(0,doorX,doorY, theLevel.getRandomFloorTile());
						theLevel.setTileAt(4,doorX,doorY, -1);
						break;
					}
					newRoomHeight = ROOM_MIN_SIZE + rand() % newRoomHeight;
					newRoomWidth = ROOM_MIN_SIZE + rand() % newRoomWidth;
					if (a==ATTEMPTS_BEFORE_FAIL - 1) {
						sprintf(buffer, "Failed putting room at: %d:%d\n", anX, anY);
						OutputDebugString(buffer);
					}
				}
			}
		}


		return true;
	}
	return false;
}

bool floorGen::doesRoomFit(int xCoord, int yCoord, int width, int height, levelManager& theLevel) {
	if (xCoord-1 < 0 || xCoord-1 > LEVEL_WIDTH || xCoord + width <= 0 || xCoord + width >= LEVEL_WIDTH ||
		yCoord-1 < 0 || yCoord-1 > LEVEL_HEIGHT || yCoord + height <= 0 || yCoord + height >= LEVEL_HEIGHT)
		return false;

	//-1 and <= to make sure that we don't directly touch another room and theirfor connect it into one massive room
	for (int x=xCoord-1; x<=xCoord + width; x++) {
		for (int y=yCoord-1; y<=yCoord + height; y++) {
			if (!isTileReplaceable(theLevel.getTileAt(0,x,y), theLevel)) 
				return false;
		}
	}
	return true;
}

bool floorGen::isTileReplaceable(int tileIndex, levelManager& theLevel) {
	return !theLevel.isWalkableTile(tileIndex);
}



void floorGen::insertRoom(int xCoord, int yCoord, int width, int height, levelManager& theLevel) {
	theLevel.roomCount++;
	for (int x=xCoord; x<xCoord + width; x++) {
		for (int y=yCoord; y<yCoord + height; y++) {
			theLevel.setTileAt(0,x,y, theLevel.getRandomFloorTile());
		}
	}
	for (int x=xCoord; x < xCoord + width; x++) {
		if (theLevel.getTileAt(4,x,yCoord-1) == bottomWall)
			theLevel.setTileAt(4,x,yCoord-1, doubleWallHorizontal);
		else
			theLevel.setTileAt(4,x,yCoord-1, topWall);

		if (theLevel.getTileAt(4,x,yCoord+height) == topWall)
			theLevel.setTileAt(4,x,yCoord+height, doubleWallHorizontal);
		else
			theLevel.setTileAt(4,x,yCoord+height, bottomWall);

	}
	for (int y=yCoord; y < yCoord + height; y++) {
		if (theLevel.getTileAt(4,xCoord-1,y) == rightWall)
			theLevel.setTileAt(4,xCoord-1,y, doubleWallVertical);
		else
			theLevel.setTileAt(4,xCoord-1,y, leftWall);

		if (theLevel.getTileAt(4,xCoord + width,y) == leftWall)
			theLevel.setTileAt(4,xCoord + width,y, doubleWallVertical);
		else
			theLevel.setTileAt(4,xCoord + width,y, rightWall);

		theLevel.setTileAt(1,xCoord-1,yCoord-1, leftTopCorner);
		theLevel.setTileAt(2,xCoord+width,yCoord+height, rightBottomCorner);
		theLevel.setTileAt(3,xCoord+width,yCoord-1, rightTopCorner);
		theLevel.setTileAt(0,xCoord-1,yCoord+height, leftBottomCorner);
	}
}

