// Blake Price
// floorGenerator.cpp
// Holds all level information
#include "levelManager.h"

//=============================================================================
// Constructor
//=============================================================================
levelManager::levelManager(EntityPlayer& player, TextDX& font) : thePlayer(player), dxFont(font)
{
	minX = maxX = minY = maxY = 0;
	level = -1;
	roomCount = 0;
	thePlayer = player;
	runMobAI = false;
	nameFont = new TextDX();
}

//=============================================================================
// Destructor
//=============================================================================
levelManager::~levelManager()
{

}

void levelManager::initialize(Game *theGame) {

	if (monsterSprites.initialize(theGame->getGraphics(),  MONSTER_IMAGE) == false) {
		throw(GameError(gameErrorNS::FATAL_ERROR, "Error loading player sprite sheet"));
	}
	if (monsterHealthSprite.initialize(theGame->getGraphics(),  MONSTER_HEALTH_IMAGE) == false) {
		throw(GameError(gameErrorNS::FATAL_ERROR, "Error loading monster health images"));
	}
	if (monsterHealthBackSprite.initialize(theGame->getGraphics(),  MONSTER_HEALTH_BACK_IMAGE) == false) {
		throw(GameError(gameErrorNS::FATAL_ERROR, "Error loading monster health images"));
	}

	if (!monsterBackHealth.initialize(theGame->getGraphics(),0,0,0,&monsterHealthBackSprite))
		throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing health back"));
	if (!monsterHealth.initialize(theGame->getGraphics(),0,0,0,&monsterHealthSprite))
		throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing health"));


	if (itemTexture.initialize(theGame->getGraphics(),  ITEMS_IMAGE) == false) {
		throw(GameError(gameErrorNS::FATAL_ERROR, "Error loading items textures"));
	}
	if (!itemImage.initialize(theGame->getGraphics(), TEXTURE_SIZE, TEXTURE_SIZE, 4, &itemTexture))
		throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing item image"));

	for (int i=0; i<MAX_ENTITY_MONSTER; i++) {
		if (entityMonsters[i].initialize(theGame, TEXTURE_SIZE, TEXTURE_SIZE, 4, &monsterSprites) == false) {
			throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing monsters"));
		}

		entityMonsters[i].setFrames(0,3);
		entityMonsters[i].setCurrentFrame(0);
		entityMonsters[i].setX(0);
		entityMonsters[i].setY(0);

	}

	if(nameFont->initialize(theGame->getGraphics(), 12, true, false, "Arial") == false)
		throw(GameError(gameErrorNS::FATAL_ERROR, "Error initializing name font"));
	//audio = theGame->getAudio();
}

void levelManager::drawMobs() {
	char  buffer[200];

	int tileX = thePlayer.worldX / 32 * -1;
	int tileY = thePlayer.worldY / 32 * -1;
	int mobX = 0;
	int mobY = 0;
	RECT r;

	nameFont->setFontColor(graphicsNS::WHITE);


	for (int i=0; i<spawnedMonsters; i++) {
		if (entityMonsters[i].getVisible() && entityMonsters[i].getX() >= tileX && entityMonsters[i].getX() <= tileX + SCREEN_WIDTH) {
			if (entityMonsters[i].getY() >= tileY && entityMonsters[i].getY() <= tileY + SCREEN_HEIGHT) {
				mobX = entityMonsters[i].getX() * TEXTURE_SIZE + thePlayer.worldX + entityMonsters[i].xOffset;
				mobY = entityMonsters[i].getY() * TEXTURE_SIZE + thePlayer.worldY+ entityMonsters[i].yOffset;
				entityMonsters[i].draw(mobX, mobY, entityMonsters[i].getSpriteData());
				dxFont.print(std::to_string((long long)i), entityMonsters[i].getX() * TEXTURE_SIZE + thePlayer.worldX + entityMonsters[i].xOffset, entityMonsters[i].getY() * TEXTURE_SIZE + thePlayer.worldY+ entityMonsters[i].yOffset);
				nameFont->print(entityMonsters[i].getName(), mobX, mobY-14);
				monsterBackHealth.setX(mobX);
				monsterBackHealth.setY(mobY-2);
				monsterBackHealth.draw();
				monsterHealth.setX(mobX+1);
				monsterHealth.setY(mobY-1);
				r = monsterHealth.getSpriteDataRect();
				r.right = monsterHealth.getWidth() / (float)entityMonsters[i].getMaxHealth(level) * entityMonsters[i].getHealth();
				monsterHealth.setSpriteDataRect(r);
				monsterHealth.draw();
			}
		}
	}
}

void levelManager::updateMobs(float frameTime) {
	for (int i=0; i<spawnedMonsters; i++) {
		entityMonsters[i].update(frameTime);
	}
}

void levelManager::aiMobs(float frameTime, bool doWalk) {
	for (int i=0; i<spawnedMonsters; i++) {

		if (entityMonsters[i].isMoving()) 
			continue;

		if (thePlayer.getPlayerX() == entityMonsters[i].getX() || thePlayer.getPlayerY() == entityMonsters[i].getY()) {
			if (thePlayer.getPlayerX() == entityMonsters[i].getX() + 1) {
				entityMonsters[i].setCurrentFrame(8);
				entityMonsters[i].setFrames(8, 8+3);
				entityMonsters[i].attackPlayer(thePlayer, level);
				continue;
			}
			else if (thePlayer.getPlayerX() == entityMonsters[i].getX() - 1) {
				entityMonsters[i].setCurrentFrame(4);
				entityMonsters[i].setFrames(4, 4+3);
				entityMonsters[i].attackPlayer(thePlayer, level);
				continue;
			}
			else if (thePlayer.getPlayerY() == entityMonsters[i].getY() + 1) {
				entityMonsters[i].setCurrentFrame(0);
				entityMonsters[i].setFrames(0, 0+3);
				entityMonsters[i].attackPlayer(thePlayer, level);
				continue;
			}
			else if (thePlayer.getPlayerY() == entityMonsters[i].getY() - 1) {
				entityMonsters[i].setCurrentFrame(12);
				entityMonsters[i].setFrames(12, 12+3); 
				entityMonsters[i].attackPlayer(thePlayer, level);
				continue;
			}
		}
		if (doWalk) {
			if (thePlayer.getPlayerX() >= entityMonsters[i].getX() - MOB_SIGHT_RADIUS && thePlayer.getPlayerX() <= entityMonsters[i].getX() + MOB_SIGHT_RADIUS &&
				thePlayer.getPlayerY() >= entityMonsters[i].getY() - MOB_SIGHT_RADIUS && thePlayer.getPlayerY() <= entityMonsters[i].getY() + MOB_SIGHT_RADIUS) {
					if (thePlayer.getPlayerX() > entityMonsters[i].getX()) {
						if (isTileUnobtrustive(entityMonsters[i].getX()+1, entityMonsters[i].getY(), i)) {
							entityMonsters[i].xMoving = MOVING_LENGTH;
							entityMonsters[i].movement = WALKING_SPEED;
							entityMonsters[i].setCurrentFrame(8);
							entityMonsters[i].setFrames(8, 8+3);
							continue;
						}
					}
					if (thePlayer.getPlayerX() < entityMonsters[i].getX()) {
						if (isTileUnobtrustive(entityMonsters[i].getX()-1, entityMonsters[i].getY(), i)) {
							entityMonsters[i].xMoving = MOVING_LENGTH;
							entityMonsters[i].movement = -WALKING_SPEED;
							entityMonsters[i].setCurrentFrame(4);
							entityMonsters[i].setFrames(4, 4+3);
							continue;
						}
					}
					if (thePlayer.getPlayerY() > entityMonsters[i].getY()) {
						if (isTileUnobtrustive(entityMonsters[i].getX(), entityMonsters[i].getY()+1, i)) {
							entityMonsters[i].yMoving = MOVING_LENGTH;
							entityMonsters[i].movement = WALKING_SPEED;
							entityMonsters[i].setCurrentFrame(0);
							entityMonsters[i].setFrames(0, 0+3);
							continue;
						}
					}
					if (thePlayer.getPlayerY() < entityMonsters[i].getY()) {
						if (isTileUnobtrustive(entityMonsters[i].getX(), entityMonsters[i].getY()-1, i)) {
							entityMonsters[i].yMoving = MOVING_LENGTH;
							entityMonsters[i].movement = -WALKING_SPEED;
							entityMonsters[i].setCurrentFrame(12);
							entityMonsters[i].setFrames(12, 12+3);
							continue;
						}
					} 
			}

			int randomDir = getRandomValidDirection(entityMonsters[i].getX(), entityMonsters[i].getY(), i);
			if (randomDir > -1) {
				switch (randomDir) {
				case DIR_LEFT:
					entityMonsters[i].xMoving = MOVING_LENGTH;
					entityMonsters[i].movement = -WALKING_SPEED;
					entityMonsters[i].setCurrentFrame(4);
					entityMonsters[i].setFrames(4, 4+3);
					continue;
				case DIR_UP:
					entityMonsters[i].yMoving = MOVING_LENGTH;
					entityMonsters[i].movement = -WALKING_SPEED;
					entityMonsters[i].setCurrentFrame(12);
					entityMonsters[i].setFrames(12, 12+3);
					continue;
				case DIR_RIGHT:
					entityMonsters[i].xMoving = MOVING_LENGTH;
					entityMonsters[i].movement = WALKING_SPEED;
					entityMonsters[i].setCurrentFrame(8);
					entityMonsters[i].setFrames(8, 8+3);
					continue;
				case DIR_DOWN:
					entityMonsters[i].yMoving = MOVING_LENGTH;
					entityMonsters[i].movement = WALKING_SPEED;
					entityMonsters[i].setCurrentFrame(0);
					entityMonsters[i].setFrames(0, 0+3);
					continue;
				}
			}
		}
	}
}	

int	levelManager::getRandomValidDirection(int x, int y, int mobId) {
	int rndDir = rand() % 4;
	for (int i=rndDir; i<4; i++)
		if (isTileUnobtrustive(getXforDir(i, x), getYforDir(i,y), mobId))
			return i;
	for (int i=0; i<rndDir; i++) 
		if (isTileUnobtrustive(getXforDir(i, x), getYforDir(i,y), mobId))
			return i;
	return -1;
}

ItemInfo levelManager::checkItemCollision() {
	for (int i=0; i<itemsOnFloor; i++) {
		if (Items[i].itemID != -1 && Items[i].xCoord == thePlayer.getPlayerX() && Items[i].yCoord == thePlayer.getPlayerY()) {
			return Items[i];//Items[i].itemID = -1;
		}
	}
	return ItemInfo();
}
void levelManager::clearItem(int x, int y) {
	for (int i=0; i<itemsOnFloor; i++) {
		if (Items[i].itemID != -1 && Items[i].xCoord == x && Items[i].yCoord == y) {
			Items[i].itemID = -1;
		}
	}
}

int levelManager::getYforDir(int dir, int y) {
	switch (dir) {
	case DIR_UP:
		return y-1;
	case DIR_DOWN:
		return y+1;
	}
	return y;
}

int levelManager::getXforDir(int dir, int x) {
	switch (dir) {
	case DIR_LEFT:
		return x-1;
	case DIR_RIGHT:
		return x+1;
	}
	return x;
}
void levelManager::preGenNext() {
	level++;
}
void levelManager::postGenNext() {
	thePlayer.setPlayerX(spawn.xCoord);
	thePlayer.setPlayerY(spawn.yCoord);
	thePlayer.stopMoving();
	spawnMonsters();
}

void levelManager::preGenPrev() {
	level--;
	if (level < 0)
		level = 0;
}
void levelManager::postGenPrev() {
	thePlayer.setPlayerX(spawn.xCoord);
	thePlayer.setPlayerY(spawn.yCoord);
	thePlayer.stopMoving();
	spawnMonsters();
}

void levelManager::spawnMonsters() {
	int mobsToSpawn = (level * 2);
	if (mobsToSpawn < MAX_ENTITY_MONSTER-1)
		mobsToSpawn += rand() % (MAX_ENTITY_MONSTER - mobsToSpawn);
	if (mobsToSpawn >= MAX_ENTITY_MONSTER)
		mobsToSpawn = MAX_ENTITY_MONSTER-1;

	spawnedMonsters = mobsToSpawn;
	Coords mobCoord;

	//DEBUG
	char  buffer[200];
	sprintf(buffer, "Spawning %d mobs\n", mobsToSpawn);
	OutputDebugString(buffer);


	for (int i=0; i<mobsToSpawn; i++) {
		entityMonsters[i].setVisible(true);
		mobCoord = getSpawnableCoords();
		entityMonsters[i].setX(mobCoord.xCoord);
		entityMonsters[i].setY(mobCoord.yCoord);
		entityMonsters[i].setHealth(entityMonsters[i].getMaxHealth(level));
		entityMonsters[i].setName("Bat");

		sprintf(buffer, "Spawning %d:%d\n", mobCoord.xCoord,mobCoord.yCoord);
		OutputDebugString(buffer);
	}
	for (int i=mobsToSpawn; i<MAX_ENTITY_MONSTER; i++) {
		entityMonsters[i].setVisible(false);
		entityMonsters[i].setX(0);
		entityMonsters[i].setY(0);
	}
}



void levelManager::drawItems() {
	int tileX = thePlayer.worldX / 32 * -1;
	int tileY = thePlayer.worldY / 32 * -1;
	for (int i=0; i<itemsOnFloor; i++) {
		if (Items[i].itemID != -1 && Items[i].xCoord >= tileX && Items[i].xCoord <= tileX + SCREEN_WIDTH) {
			if (Items[i].yCoord >= tileY && Items[i].yCoord <= tileY + SCREEN_HEIGHT) {
				itemImage.setX(Items[i].xCoord * TEXTURE_SIZE + thePlayer.worldX);
				itemImage.setY(Items[i].yCoord * TEXTURE_SIZE + thePlayer.worldY);
				itemImage.setCurrentFrame(Items[i].itemTileIndex);
				itemImage.draw();
			}
		}
	}
}

Coords levelManager::getSpawnableCoords() {
	Coords theCoords;
	int xCoord = minX + rand() % maxX;
	int yCoord = minY + rand() % maxY;
	for (int i=0; i<1000; i++) { //Just to prevent infinite looping. Should not ever happen, testing shows 1-6 loops finds a valid area.
		if (isTileUnobtrustive(xCoord,yCoord) && isTileUnobtrustive(xCoord+1,yCoord) && isTileUnobtrustive(xCoord-1,yCoord) &&
			isTileUnobtrustive(xCoord,yCoord+1) && isTileUnobtrustive(xCoord,yCoord-1) && isTileUnobtrustive(xCoord-1,yCoord-1) &&
			isTileUnobtrustive(xCoord-1,yCoord+1) && isTileUnobtrustive(xCoord+1,yCoord-1) && isTileUnobtrustive(xCoord+1,yCoord+1)) {

				theCoords.xCoord = xCoord;
				theCoords.yCoord = yCoord;
				return theCoords;
		}
		xCoord = minX + rand() % maxX;
		yCoord = minY + rand() % maxY;
	}
	theCoords.xCoord = 0;
	theCoords.yCoord = 0;
	return theCoords; //Failed
}

Coords levelManager::getItemSpawnableCoords() {
	Coords theCoords;
	int xCoord = minX + rand() % maxX;
	int yCoord = minY + rand() % maxY;
	for (int i=0; i<1000; i++) {
		if (isTileUnobtrustive(xCoord,yCoord)) { //We don't care if the items are blocked on any side
			theCoords.xCoord = xCoord;
			theCoords.yCoord = yCoord;
			return theCoords;
		}
		xCoord = minX + rand() % maxX;
		yCoord = minY + rand() % maxY;
	}
	theCoords.xCoord = 0;
	theCoords.yCoord = 0;
	return theCoords; //Failed
}

bool levelManager::isTileUnobtrustive(int x, int y, int mobId) {
	return isWalkableTile(getTileAt(0,x,y)) && getTileAt(OBSTRUCTION_LAYER,x,y) == -1 && !isEntityAt(x,y, mobId) && (mobId == -1 || ((stairs.xCoord != x || stairs.yCoord != y) && (stairsUp.xCoord != x || stairsUp.yCoord != y)));
}

bool levelManager::isWallAt(int x, int y) {
	return getTileAt(0, x, y) == 48 || getTileAt(1, x, y) >= 0 || getTileAt(2, x, y) >= 0 || getTileAt(3, x, y) >= 0 || getTileAt(4, x, y) >= 0;
}

bool levelManager::isWalkableTile(int tileID) {
	return tileID == floorTile[0] || tileID == floorTile[1] || tileID == floorTile[2] || tileID == floorTile[3] ||
		tileID == floorTile[4] || tileID == floorTile[5] || tileID == floorTile[6] || tileID == floorTile[7] ||
		tileID == floorTile[8] || tileID == floorTile[9] || tileID == floorTile[10] || tileID == floorTile[11]; //ToDo: Add more floor tiles
}

int levelManager::getRandomFloorTile() {
	return floorTile[rand() % 12];
}
bool levelManager::isEntityAt(int x, int y, int mobId) {

	for (int i=0; i<spawnedMonsters; i++) 
		if (i != mobId)
			if (entityMonsters[i].getX() == x && entityMonsters[i].getY() == y && !entityMonsters[i].isMoving())
				return true;
			else if (entityMonsters[i].isMoving()) //Check if entity is already moving to this square
				if (entityMonsters[i].xMoving > 0 && entityMonsters[i].getY() == y && entityMonsters[i].getX() + entityMonsters[i].movement == x)
					return true;
				else if (entityMonsters[i].yMoving > 0 && entityMonsters[i].getX() == x && entityMonsters[i].getY() + entityMonsters[i].movement == y)
					return true;
	if (mobId != -1) {
		if (thePlayer.getPlayerX() == x && thePlayer.getPlayerY() == y && !thePlayer.isMoving())
			return true;
		else if (thePlayer.isMoving()) //Check if entity is already moving to this square
			if (thePlayer.getXMoving() > 0 && thePlayer.getPlayerY() == y && thePlayer.getPlayerX() + thePlayer.getMovement() * -1 == x)
				return true;
			else if (thePlayer.getYMoving() > 0 && thePlayer.getPlayerX() == x && thePlayer.getPlayerY() + thePlayer.getMovement() * -1 == y)
				return true;
	}
	return false;
}
EntityMonster *levelManager::getEntityMonsterAt(int x, int y) {
	for (int i=0; i<spawnedMonsters; i++) 
		if (entityMonsters[i].getX() == x && entityMonsters[i].getY() == y)
			return &entityMonsters[i];


	return NULL;
}

bool levelManager::playerAttackMonster(EntityMonster& aMonster) {
	if (thePlayer.canAttack()) {
		int temp = (rand() % thePlayer.getPlayerStr() * 0.2) + thePlayer.getPlayerStr() * .9;
		aMonster.getAudio()->playCue(PvM);
		infochat::addChat("You hit the " + aMonster.getName() + " for " + std::to_string(temp) + "!");
		aMonster.setHealth(aMonster.getHealth() - thePlayer.getPlayerStr());
		if (aMonster.getHealth() <= 0) {
			thePlayer.addExp(aMonster.getDroppedExp(level));
			aMonster.setX(-100);
			aMonster.setY(-100);
			aMonster.setVisible(false);
		}
		return true;
	}
	return false;
}

int levelManager::getTileAt(int layer, int x, int y) {
	if (layer >= 0 && layer < LEVEL_LAYERS && x >= 0 && x < LEVEL_WIDTH && y >= 0 && y < LEVEL_HEIGHT) 
		return mapTiles[layer][x][y];  
	else 
		return -1; 
}

void levelManager::setTileAt(int layer, int x, int y, int value) {
	if (layer >= 0 && layer < LEVEL_LAYERS && x >= 0 && x < LEVEL_WIDTH && y >= 0 && y < LEVEL_HEIGHT) 
		mapTiles[layer][x][y] = value;  
}

int levelManager::getFogTileAt(int x, int y) {
	if (x >= 0 && x < LEVEL_WIDTH && y >= 0 && y < LEVEL_HEIGHT) 
		return fogLayer[x][y];  
	else 
		return -1; 
}

void levelManager::setFogTileAt(int x, int y, int value) {
	if (x >= 0 && x < LEVEL_WIDTH && y >= 0 && y < LEVEL_HEIGHT) 
		fogLayer[x][y] = value;  
}

void levelManager::clearFog() {
	for (int x=thePlayer.getPlayerX() - FOG_VISIBLITY; x<thePlayer.getPlayerX() + FOG_VISIBLITY; x++) {
		for (int y=thePlayer.getPlayerY() - FOG_VISIBLITY; y<thePlayer.getPlayerY() + FOG_VISIBLITY; y++) {
			setFogTileAt(x,y,-1);
		}
	}
}

void levelManager::setMinY(int y) {
	if (y < minY) {
		minY = y;
	}
}

void levelManager::setMaxY(int y){
	if (y > maxY) {
		maxY = y;
	}
}

void levelManager::setMinX(int x) {
	if (x < minX) {
		minX = x;
	}
}

void levelManager::setMaxX(int x){
	if (x > maxX) {
		maxX = x;
	}
}