#ifndef _LVLMNG_H           // Prevent multiple definitions if this 
#define _LVLMNG_H           // file is included in more than one place
#define WIN32_LEAN_AND_MEAN

#include <string>
#include <sstream>
#include "constants.h"
#include "EntityPlayer.h"
#include "EntityMonster.h"
#include "Coords.h"
#include "ItemInfo.h"

//=============================================================================
// Holds all of the information for the current level
//=============================================================================
class levelManager
{
protected:
	//Audio   *audio;
private:
	int mapTiles[LEVEL_LAYERS][LEVEL_WIDTH][LEVEL_HEIGHT];
	int fogLayer[LEVEL_WIDTH][LEVEL_HEIGHT];
	EntityMonster entityMonsters[MAX_ENTITY_MONSTER];
	

	int spawnedMonsters;
	

	TextureManager monsterSprites;
	TextureManager monsterHealthSprite;
	TextureManager monsterHealthBackSprite;
	TextureManager itemTexture;
	Image itemImage;
	Image monsterBackHealth;
	Image monsterHealth;
	EntityPlayer& thePlayer;
	TextDX& dxFont;
	
	


public:

	
	// Constructor
	levelManager(EntityPlayer& player, TextDX& font);
	// Destructor
	virtual ~levelManager();

	void levelManager::initialize(Game *theGame);

	void preGenNext();
	void postGenNext();
	void preGenPrev();
	void postGenPrev();

	
	void drawItems();

	void spawnMonsters();
	void drawMobs();
	void updateMobs(float frameTime);
	void aiMobs(float frameTime, bool doWalk);
	EntityMonster *getEntityMonsterAt(int x, int y);

	bool playerAttackMonster(EntityMonster& aMonster);

	Coords getSpawnableCoords();
	Coords getItemSpawnableCoords();

	int getTileAt(int layer, int x, int y);
	void setTileAt(int layer, int x, int y, int value);
	int getFogTileAt(int x, int y);
	void setFogTileAt(int x, int y, int value);
	bool isTileUnobtrustive(int x, int y, int mobId = -2);
	bool isWallAt(int x, int y);
	bool isEntityAt(int x, int y, int mobId);
	int	getRandomValidDirection(int x, int y, int mobId = -2);
	int getYforDir(int dir, int y);
	int getXforDir(int dir, int x);
	void clearFog();
	bool isWalkableTile(int tileID);
	int getRandomFloorTile();
	ItemInfo checkItemCollision();
	void clearItem(int x, int y);

	void setMinY(int y);
	void setMaxY(int y);
	void setMinX(int x);
	void setMaxX(int x);

	int minX;
	int minY;
	int maxX;
	int maxY;

	Coords stairs;
	Coords stairsUp;
	Coords spawn;
	ItemInfo Items[MAX_ITEMS];

	int roomCount;
	int itemsOnFloor;

	int level;

	bool runMobAI;
	TextDX  *nameFont; 



};

#endif